/*
 *  =============================================================
 *  DemoArrays.java: Demo features of arrays in Java
 *
 *  An array is a structure that holds multiple values of the 
 *  same data type.
 *
 *  Written By: Mark Austin                          October 2003
 *  =============================================================
 */

package basics;

import java.text.*;

public class DemoArrays {

   // Define NoColums to be a "constant" .... (see use below)

   public final static int NoColumns = 6;

   // main method : this is where the program execution begins.

   public static void main ( String [] args ) {
       int i;

       // [a] Create and populate a one dimensional array of
       //     five integers

       int [] iA  = new int[5];
       for ( i = 0; i <= 4; i = i + 1)
            iA [i] = i;

       // [b] Print contents of iA

       System.out.println("Array: iA");
       for ( i = 0; i <= 4; i = i + 1)
            System.out.println( iA [i] );

       // [c] Retrieve and print the length of iA...

       System.out.println("");
       System.out.println("Length of iA = " + iA.length );

       // [d] Create and initialize a one dimensional array of
       //     twenty floating point nos ....

       float [] fA  = new float[ 20 ];
       for ( i = 0; i < fA.length; i = i + 1)
            fA [i] = (float) i;

       // [e] Print contents of "fA" organized into six columns...


       System.out.println("");
       System.out.println("Array: fA (standard printout)");
       for ( i = 1; i <= fA.length; i = i + 1) {
             System.out.print(" " + fA[i-1] );
             if (i % NoColumns == 0 || i == fA.length ) 
                 System.out.println("");
       }

       // [f] Print contents of "fA" formatted and organized
       //     into six columns...

       DecimalFormat formatOutput = new DecimalFormat ("00.00");

       System.out.println("");
       System.out.println("Array: fA (formatted output)");
       for ( i = 1; i <= fA.length; i = i + 1) {

             // Format and print array element ....

             String output1 = formatOutput.format( fA [i-1] );
             System.out.print(" " + output1 );

             // Print end of line ....

             if (i % NoColumns == 0 || i == fA.length ) 
                 System.out.println("");
       }

       // [g] Create, initialize, and print an array of Strings

       String [] saNames = { "Coronet Peak",
                                 "Heavenly",
                                     "Vail",
                       "Whistler/Blackcomb" };

       System.out.println("");
       System.out.println("Length of saNames = " + saNames.length );

       System.out.println("");
       System.out.println("Array: saNames");
       for ( i = 0; i < saNames.length; i = i + 1)
            System.out.println( saNames [i] );

       // [h] Create, initialize and print a two-dimensional array
       //     of strings....they are arrays-of-arrays.

       String skiResorts [][] = { {"Coronet Peak",      "New Zealand" },
                                      {"Heavenly",       "California" },
                                    {"Grouse Mtn",        "Vancouver" },
                                      {"Mt Baker",       "Washington" },
                                          {"Vail",         "Colorado" },
                            {"Whistler/Blackcomb", "British Columbia" } };

       System.out.println("");
       System.out.println("No of skiResorts = " + skiResorts.length );
       System.out.println("Length of first row  = " + skiResorts[0].length );
       System.out.println("Length of second row = " + skiResorts[1].length );

       System.out.println("");
       System.out.println("Array: skiResorts ");
       for ( i = 0; i < skiResorts.length; i = i + 1)
            System.out.println( "  " + skiResorts [i][0] +
                                " is in " + skiResorts [i][1] );
   }
}
