/*
 *  =====================================================================
 *  DemoMathAngle.java: Compute angle from (x,y) coordinate positions ...
 *  
 *  Modified: Mark Austin                                  February, 2022  
 *  =====================================================================
 */

package basics;

import java.lang.Math;

public class DemoMathAngle {
    public static void main( String args[] ) {
        double dX, dY, angle;

        // Part 1. Naive Implementation ....

        System.out.println("Naive Angle Calculation ");
        System.out.println("======================= ");

        // First quadrant ....

        dX = 1.0; dY = 0.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = 1.0; dY = 1.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = 0.0; dY = 1.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );

        // Second quadrant ....
        dX = -1.0; dY = 1.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = -1.0; dY = 0.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );

        // Third quadrant ....
        dX = -1.0; dY = -1.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );

        // Fourth quadrant ....
        dX =  1.0; dY = -1.0; angle = Math.atan( dY/dX );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );

        // Part 2. Better Implementation ....

        System.out.println("Quadrant Adjusted Calculation ");
        System.out.println("============================= ");

        dX = 1.0; dY = 0.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = 1.0; dY = 1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = 0.0; dY = 1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = -1.0; dY = 1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = -1.0; dY = 0.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX = -1.0; dY = -1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX =  0.0; dY = -1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX =  1.0; dY = -1.0; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );
        dX =  1.0; dY = -0.1; angle = getAngle ( dX, dY );
        System.out.println ( "(x,y) = (" + dX + "," + dY + ") angle = " + angle );

    }

    // ====================================================
    // Compute angle for coordinates in four quadrants ....
    // ====================================================

    static double getAngle( double dX, double dY ) {
        double angle = 0.0;

        if ( dY >= 0.0 && dX >= 0.0 )
             angle = Math.atan( dY/dX );
        if ( dY >= 0.0 && dX < 0.0 )
             angle = Math.PI + Math.atan( dY/dX );
        if ( dY  < 0.0 && dX < 0.0 )
             angle = Math.PI + Math.atan( dY/dX );
        if ( dY  < 0.0 && dX >= 0.0 )
             angle = 2*Math.PI + Math.atan( dY/dX );

        return angle;

    }
}
