/* 
 *  =====================================================================
 *  DemoCellLinesScanner03.java: Use scanner to read in various data types
 *  from cell lines file:
 *  
 *                                                         November, 2019
 *  =====================================================================
 */ 

package io;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Scanner;
import java.util.ArrayList;

public class DemoCellLinesScanner03 {
  public static void main(String args[]) throws IOException {
     int      i;
     double   d;
     boolean  b;
     String str;
     String delimiters = "\\s+|,\\s*|\\.\\s*";

     // Write string to "DemoCellLinesScanner03.txt" output file ...

     FileWriter fout = new FileWriter("DemoCellLinesScanner03.txt");
     fout.write("Testing Cell Lines Scanner ... ");
     fout.close();

     // Create reader for "test.txt" ...

     FileReader fin = new FileReader("data/cell-gdsc-data.txt");

     // Create scanner to read on file input stream ... 

     Scanner src = new Scanner(fin);

     // Process header line ...

     String header   = src.nextLine();
     String[] tokens = header.split( delimiters ); 

     // Build arraylist of cell lines ...

     ArrayList cellLines = new ArrayList();
     for (int k = 1; k < tokens.length; k = k + 1) {
         cellLines.add( tokens[k] );
     }

     // Print details of cell lines ...

     System.out.printf("Header Line ...\n");
     System.out.printf("=========================================== ...\n");
     for (int k = 1; k <= cellLines.size(); k = k + 1) {
         System.out.printf("Cell line %2d: id --> %s ...\n", k, (String) cellLines.get(k-1) ); 
     }
     System.out.printf("=========================================== ...\n");

     // Iterate over lines...
 
     i = 1;
     while (src.hasNextLine()) {

        // Read and split line into tokens ...

        String line = src.nextLine();
        tokens = line.split( delimiters ); 

        // Process tokens within line ...

        System.out.printf("\n"); 
        System.out.printf("Drug %2d: %s ...\n", i, tokens[0]); 
        System.out.printf("-----------------------------------------------\n");

        for (int j = 1; j < tokens.length; j = j + 1) {
            String item = tokens[j];

            boolean isNumber = isNumeric( item );
            if (isNumber == true ) {
               int evidenceLevel = Integer.valueOf( item );
               switch (evidenceLevel) {
                   case 0:
                        System.out.printf("Row %2d: Evidence(%d): Strong evidence drug kills cell line %s ...\n", 
                                           j, evidenceLevel, cellLines.get(j-1) ); 
                        break;
                   case 1:
                        System.out.printf("Row %2d: Evidence(%d): Weak evidence drug kills cell line %s ...\n", 
                                           j, evidenceLevel, cellLines.get(j-1) ); 
                        break;
                   case 2:
                        System.out.printf("Row %2d: Evidence(%d): No evidence that drug works on %s ...\n", 
                                           j, evidenceLevel, cellLines.get(j-1) ); 
                        break;
                   default:
                        break;
               }
            } else if (item.equals("NA") == true ) 
               System.out.printf("Row %2d: NA (no info available) for %s ...\n", j, cellLines.get(j-1) ); 
            else {
               System.out.printf("Row %2d: ERROR !! ...\n", j ); 
            }
        }

        System.out.printf("-----------------------------------------------\n");
        i = i + 1;
     }

     // Close input file ...

     fin.close();
  }

  public static boolean isNumeric(String strNum) {
     return strNum.matches("-?\\d+(\\.\\d+)?");
  }
}

