/* 
 *  ========================================================================
 *  DemoTimeSeriesBarChart01.java: Create time series bar chart plot.
 * 
 *  Written by: Mark Austin                                    November 2021 
 *  ========================================================================
 */ 

package jfreechart;

import java.awt.Color;
import java.awt.GridLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;

import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.DateTickUnit;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;

import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.category.BarRenderer;

import org.jfree.chart.ui.RectangleInsets;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.time.Year;
import org.jfree.data.time.Month;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYBarDataset;
import org.jfree.data.xy.XYDataset;

import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;

import java.util.List;
import java.util.Date;

public class DemoTimeSeriesBarChart01 extends JFrame {

    // Constructor meethods ...

    public DemoTimeSeriesBarChart01() {
       try {
          initUI();
       } catch(IOException e) {}
    }

    public DemoTimeSeriesBarChart01( String name ) {
       super (name);
       try {
          initUI();
       } catch(IOException e) {}

    }

    // Initialize datasets ...

    private void initUI() throws IOException {
        XYDataset dataset = createDataset("level");
        JFreeChart chart  = createBarChart(dataset, "level vs date" );

        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        chartPanel.setBackground(Color.white);
        
        add(chartPanel);

        pack();
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    // ================================================================
    // Create dataset from data/stocks.csv
    // ================================================================

    private static TimeSeriesCollection createDataset(String title) {

        TimeSeries s1 = new TimeSeries(title);

        // Dates in 2010 ....

        Date dateItem = new Date("Jan 1 2010");
        s1.add( new Month( dateItem ), 1.0 );

        dateItem = new Date("Feb 1 2010");
        s1.add( new Month( dateItem ), 0.5 );

        dateItem = new Date("March 1 2010");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("April 1 2010");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("May 1 2010");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("June 1 2010");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("July 1 2010");
        s1.add( new Month( dateItem ), 1.0 );

        dateItem = new Date("Aug 1 2010");
        s1.add( new Month( dateItem ), 1.0 );

        dateItem = new Date("Sept 1 2010");
        s1.add( new Month( dateItem ), 5.0 );

        dateItem = new Date("Oct 1 2010");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("Nov 1 2010");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("Dec 1 2010");
        s1.add( new Month( dateItem ), 4.0 );

        // Dates in 2011 ....

        dateItem = new Date("Jan 1 2011");
        s1.add( new Month( dateItem ), 1.0 );

        dateItem = new Date("Feb 1 2011");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("March 1 2011");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("April 1 2011");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("May 1 2011");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("June 1 2011");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("July 1 2011");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("Aug 1 2011");
        s1.add( new Month( dateItem ), 4.0 );

        dateItem = new Date("Sept 1 2011");
        s1.add( new Month( dateItem ), 5.0 );

        dateItem = new Date("Oct 1 2011");
        s1.add( new Month( dateItem ), 3.0 );

        dateItem = new Date("Nov 1 2011");
        s1.add( new Month( dateItem ), 2.0 );

        dateItem = new Date("Dec 1 2011");
        s1.add( new Month( dateItem ), 4.0 );

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(s1);

     // dataset.setDomainIsPointsInTime(false);

        return dataset;
    }

    // =======================================================
    // Create bar chart ....
    // =======================================================

    private JFreeChart createBarChart( XYDataset dataset, String title ) {

        IntervalXYDataset idataset = new XYBarDataset(dataset, 0.5);

        JFreeChart chart = ChartFactory.createXYBarChart(
                title,              // title
                "Date",             // x-axis label
                true,               // date axis?
                "Level",   // y-axis label
                idataset,           // interval dataset
                PlotOrientation.VERTICAL,       // orientation
                true,               // create legend?
                true,               // generate tooltips?
                false               // generate URLs?
        );
 
        // Set chart styles

        chart.setBackgroundPaint(Color.white);

        // Set plot styles

        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.white);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.black);
        plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));

        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
                
        // Set series line styles

        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setDefaultShapesVisible(true);
            renderer.setDefaultShapesFilled(true);
        }
        
        // Set range axis style

        NumberAxis rangeaxis = (NumberAxis) plot.getRangeAxis();
        rangeaxis.setTickUnit( new NumberTickUnit(1.0) );

        // Set date axis style

        DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("MMM-yyyy"));

        return chart;
    }

    public static void main(String[] args) {
       EventQueue.invokeLater(() -> {
          DemoTimeSeriesBarChart01 ex = new DemoTimeSeriesBarChart01("Bar Chart Test");
          ex.setVisible(true);
       });
    }
}
