/* 
 *  ========================================================================
 *  DemoTimeSeriesChart01.java: Simple timeseries chart ...
 *  ========================================================================
 */ 

package jfreechart;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.ui.RectangleInsets;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.data.time.Day;
import org.jfree.data.time.Month;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;

public class DemoTimeSeriesChart01 extends JFrame {

    // Constructor meethods ...

    public DemoTimeSeriesChart01() {
       initUI();
    }

    public DemoTimeSeriesChart01( String name ) {
       super (name);
       initUI();
    }

    // Initialize datasets ...

    private void initUI() {
        XYDataset dataset = createDataset();
        JFreeChart chart  = createChart(dataset);

        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        chartPanel.setBackground(Color.white);
        
        add(chartPanel);

        pack();
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    // Crete dataset ....

    private XYDataset createDataset() {

        TimeSeries s1 = new TimeSeries("European Index Trust");
          s1.add(new Month(2, 2001), 181.8);
          s1.add(new Month(3, 2001), 167.3);
          s1.add(new Month(4, 2001), 153.8);
          s1.add(new Month(5, 2001), 167.6);
          s1.add(new Month(6, 2001), 158.8);
          s1.add(new Month(7, 2001), 148.3);
          s1.add(new Month(8, 2001), 153.9);
          s1.add(new Month(9, 2001), 142.7);
          s1.add(new Month(10, 2001), 123.2);
          s1.add(new Month(11, 2001), 131.8);
          s1.add(new Month(12, 2001), 139.6);
          s1.add(new Month(1, 2002), 142.9);
          s1.add(new Month(2, 2002), 138.7);
          s1.add(new Month(3, 2002), 137.3);
          s1.add(new Month(4, 2002), 143.9);
          s1.add(new Month(5, 2002), 139.8);
          s1.add(new Month(6, 2002), 137.0);
          s1.add(new Month(7, 2002), 132.8);

        TimeSeries s2 = new TimeSeries("UK Index Trust");
          s2.add(new Month(2, 2001), 129.6);
          s2.add(new Month(3, 2001), 123.2);
          s2.add(new Month(4, 2001), 117.2);
          s2.add(new Month(5, 2001), 124.1);
          s2.add(new Month(6, 2001), 122.6);
          s2.add(new Month(7, 2001), 119.2);
          s2.add(new Month(8, 2001), 116.5);
          s2.add(new Month(9, 2001), 112.7);
          s2.add(new Month(10, 2001), 101.5);
          s2.add(new Month(11, 2001), 106.1);
          s2.add(new Month(12, 2001), 110.3);
          s2.add(new Month(1, 2002), 111.7);
          s2.add(new Month(2, 2002), 111.0);
          s2.add(new Month(3, 2002), 109.6);
          s2.add(new Month(4, 2002), 113.2);
          s2.add(new Month(5, 2002), 111.6);
          s2.add(new Month(6, 2002), 108.8);
          s2.add(new Month(7, 2002), 101.6);

        // Assemble collection of x-y lines ...

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(s1);
        dataset.addSeries(s2);

        return dataset;
    }

    private JFreeChart createChart(final XYDataset dataset) {

        JFreeChart chart = ChartFactory.createTimeSeriesChart(
            "General Unit Trust Prices",  // title
            "Date",             // x-axis label
            "Price Per Unit",   // y-axis label
            dataset,            // data
            true,               // create legend?
            true,               // generate tooltips?
            false               // generate URLs?
        );

        chart.setBackgroundPaint(Color.white);

        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.white);
        plot.setDomainGridlinePaint(Color.black);
        plot.setRangeGridlinePaint(Color.black);
        plot.setAxisOffset( new RectangleInsets(5.0, 5.0, 5.0, 5.0) );
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
       
        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setSeriesPaint(0, Color.RED);
            renderer.setSeriesStroke(0, new BasicStroke(2.0f));
            renderer.setSeriesPaint(1, Color.BLUE);
            renderer.setSeriesStroke(1, new BasicStroke(2.0f));
        }

        DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("MMM-yyyy"));

        chart.getLegend().setFrame(BlockBorder.NONE);
        chart.setTitle(new TextTitle("UK and European Index Funds", new Font("Serif", Font.BOLD, 18)));

        return chart;
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(() -> {
            DemoTimeSeriesChart01 ex = new DemoTimeSeriesChart01("UK and European Index Fund Time Series");
            ex.setVisible(true);
        });
    }
}
