/*
 *  =======================================================================
 *  TestMatrix.java: Exercise methods in the Matrix class ....
 *
 *  Written by: Mark Austin                                  September 2021
 *  =======================================================================
 */

package objects;

import java.lang.Math;
import java.util.*;
import java.io.*;
import java.text.*;

public class TestMatrix {
    public static void main(String[] args) {

        System.out.printf("Enter TestMatrix.main()                ... \n");
        System.out.printf("====================================== ... \n");

        System.out.printf("--- Create small (3x3) matrix ... \n");

        double[][] d = { { 1, 2, 3 }, { 4, 5, 6 }, { 9, 1, 3} };
        Matrix D = new Matrix(d);

        D.print();        
        System.out.println();

        System.out.printf("--- Create A = (5x5) matrix containing random elements ... \n");

        Matrix A = Matrix.random(5, 5);
        A.print(); 
        System.out.println();

        System.out.printf("--- Swap row 1 and 2 ... \n");

        A.swap(1, 2);
        A.print(); 
        System.out.println();

        System.out.printf("--- Compute and print A transpose ... \n");

        Matrix B = A.transpose();
        B.print(); 
        System.out.println();

        System.out.printf("--- Create C = (5x5) identity matrix ... \n");

        Matrix C = Matrix.identity(5);
        C.print(); 
        System.out.println();

        System.out.printf("--- Compute and print A + B ... \n");

        A.plus(B).print();
        System.out.println();

        System.out.printf("--- Compute and print B * A ... \n");

        B.times(A).print();
        System.out.println();

        // shouldn't be equal since AB != BA in general    

        System.out.printf("--- Check: Is AB == BA? ... \n");

        System.out.println(A.times(B).equals(B.times(A)));
        System.out.println();

        // Create a large identity martix

        System.out.printf("--- Create large (10x10) identity matrix ... \n");

        Matrix large = Matrix.identity(10);
        large.setName("Identity!"); 
        large.print(); 

        // Solve a simple set of equations 

        System.out.printf("--- Solve simple set of equations ... \n");

        Matrix a = new Matrix ( "A", 3, 3 );
        a.setElement( 0, 0,  1 );
        a.setElement( 0, 1,  2 );
        a.setElement( 0, 2,  3 );

        a.setElement( 1, 0,  5 );
        a.setElement( 1, 1,  6 );
        a.setElement( 1, 2,  7 );

        a.setElement( 2, 0,  2 );
        a.setElement( 2, 1,  4 );
        a.setElement( 2, 2,  8 );
        a.print();

        Matrix b = new Matrix ( "B", 3, 1 );
        b.setElement( 0, 0,  6 );
        b.setElement( 1, 0, 18 );
        b.setElement( 2, 0, 14 );
        b.print();

        // Compute solution to equations ....

        System.out.printf("--- Compute solutions to matrix ... \n");

        Matrix x = a.solve( b );
        x.setName("x");

        // Print and check result ...

        System.out.printf("--- Print and check result ... \n");

        x.print();
        a.times(x).print();

        System.out.printf("====================================== ... \n");
        System.out.printf("Finished !! ... \n");
    }
}
