/*
 *  =================================================================
 *  DemoDateFormatOutput01.java: Introduction to formatting of dates
 *  with java.util.Date and java.time.format.DateTimeFormatter.
 *  
 *  Written by: Mark Austin                             November 2021  
 *  =================================================================
 */

package time;

import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;

public class DemoDateFormatOutput01 {
    public static void main(String[] args) {

        System.out.printf("Enter DemoDateFormatOutput01.main() ... \n" );
        System.out.printf("===================================================== \n" );

        // Part 1. Print current date and time in default format...

        // Use the Calendar class to get a snapshot of this precise moment.
        // Convert to a Date and then to a string.

        Calendar thisInstant = Calendar.getInstance(); 
        Date rightNow = thisInstant.getTime(); 
        String ex1    = rightNow.toString();

        System.out.printf("--- Part 1. Current time/date (default format): %s ...\n", ex1);

        // Part 2.  Get the current date and time, and print it out in a
        // specified way as described by the SimpleDateFormat that we've
        // designated below.

        // An alternative way to get the current date and time that doesn't
        // involve the Calendar class.

        Date now = new Date(System.currentTimeMillis());
        SimpleDateFormat sdfEx2 = new SimpleDateFormat("EEEE, MMM d, yyyy hh:mm aaa");
        String ex2 = sdfEx2.format(now);

        System.out.printf("--- Part 2. Current time and date (custom format): %s ...\n", ex2);

        // Part 3. Print today's date (only) in a specified way as described
        // by the alternative SimpleDateFormat that we've designated.

        SimpleDateFormat sdfEx3 = new SimpleDateFormat("MMMM d, yyyy");
        String ex3 = sdfEx3.format(now);

        System.out.printf("--- Part 3. Today's date (custom format 1): %s ...\n", ex3);

        // Part 4. Print today's date (only) in yet another format.

        SimpleDateFormat sdfEx4 = new SimpleDateFormat("MM/dd/yyyy");
        String ex4 = sdfEx4.format(now);

        System.out.printf("--- Part 4. Today's date (custom format 2): %s ...\n", ex4);

        // Part 5. Craft a specific date as a Date object.

        Calendar cal = Calendar.getInstance();
        cal.set(1975, Calendar.DECEMBER, 1 );
        Date birthDate = cal.getTime();

        // We'll reuse the SimpleDateFormat from above.

        String ex5 = sdfEx4.format(birthDate);

        System.out.printf("--- Part 5. Birth date: %s ...\n", ex5);

        // Part 6. Generate output with builtin formatting style ...

        System.out.printf("--- Part 6. Exercise date-time formatting styles ...\n");

        LocalDate holiday = LocalDate.of(2021, 11, 24);
        String ex6 = DateTimeFormatter.ofLocalizedDate(FormatStyle.FULL).format( holiday );
        System.out.printf("---   FormatStyle.FULL: %s ...\n", ex6 );
        String ex7 = DateTimeFormatter.ofLocalizedDate(FormatStyle.LONG).format( holiday );
        System.out.printf("---   FormatStyle.LONG: %s ...\n", ex7 );
        String ex8 = DateTimeFormatter.ofLocalizedDate(FormatStyle.MEDIUM).format( holiday );
        System.out.printf("---   FormatStyle.MEDIUM: %s ...\n", ex8 );
        String ex9 = DateTimeFormatter.ofLocalizedDate(FormatStyle.SHORT).format( holiday );
        System.out.printf("---   FormatStyle.SHORT: %s ...\n", ex9 );

        // Part 7. Compare two dates. We'll reuse the Calendar instance
        // from Part 5 to craft two different Dates.

        System.out.printf("--- Part 7. Check scheduling of summer vacation ...\n");

        cal.set(2021, Calendar.JULY, 1);
        Date startSummerVacation = cal.getTime();
        cal.set(2021, Calendar.JUNE, 1);
        Date endSummerVacation = cal.getTime();

        System.out.printf("---   Summer vacation starts: %s ...\n", startSummerVacation.toString()) ;
        System.out.printf("---   Summer vacation end: %s ...\n", endSummerVacation.toString()) ;

        // We'll use the before() method to compare the dates (note that there's
        // also an after() method and an equals() method ...).

        if (startSummerVacation.before( endSummerVacation) ) 
            System.out.printf("--- Check: Dates are in correct order ...\n" );
        else 
            System.out.printf("--- Check: Dates are in incorrectly ordered !!!\n" );

        System.out.printf("===================================================== \n" );
        System.out.printf("Finished !! \n" );
    }
}
